/*
 * dn2ethaddr.c
 *
 * Version 1.0 (C) 1998 Steven Whitehouse <SteveW@ACM.org>
 * Modifications by Kenn Humborg <<kenn@avalon.wombat.ie>
 *
 * This code is distributed under the terms of the GNU Copyleft.
 * The author disclaims responsibility for everything: use at your own peril!
 * Please email constructive comments only to the above email address.
 *
 * This program is intended to simplify the setting up of ethernet cards
 * to work with DECnet. You can use it in scripts by doing something like
 * the following:
 *
 * ifconfig eth0 up ${IPADDR} hw ether `dn2ethaddr -n ${DNADDR}`
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <getopt.h>
#include <sysexits.h>

#define VERSION "dn2ethaddr: Version 1.0 (C) 1998 Steven Whitehouse <SteveW@ACM.org>"

void version(void)
{
	fprintf(stderr, VERSION "\n");
}

void usage(void)
{
	fprintf(stderr, "Usage:\n"
			"dn2ethaddr [-n] <dnaddr>\n"
			"dn2ethaddr [-hV]\n"
			"Options:\n"
			"        h - Print this help message\n"
			"        n - Don't print a newline\n"
			"        V - Print version and exit\n"
			" <dnaddr> - A DECnet address in the format area.node\n"
			);
}

int main(int argc, char *argv[])
{
	int opt;
	int newline = 1;
	unsigned short addr;
	int node, area;

	while((opt = getopt(argc, argv, "Vhn")) != EOF) {
		switch(opt) {
			case 'V':
				version();
				exit(EX_USAGE);
			case '?':
			default:
				fprintf(stderr, "Invalid option\n");
			case 'h':
				version();
				usage();
				exit(EX_USAGE);
			case 'n':
				newline = 0;
				break;
		}
	}

	if ((argc - optind) != 1) {
		fprintf(stderr, "Invalid number of argments\n");
		version();
		usage();
		exit(EX_USAGE);
	}

	if (sscanf(argv[optind], "%d.%d", &area, &node) != 2) {
		fprintf(stderr, "DECnet address not recognised\n");
		exit(EX_DATAERR);
	}

	if ((area > 63) || (area < 0)) {
		fprintf(stderr, "DECnet area is between 0 and 63 inclusive\n");
		exit(EX_DATAERR);
	}

	if ((node > 1023) || (node < 0)) {
		fprintf(stderr, "DECnet node is between 0 and 1023 inclusive\n");
		exit(EX_DATAERR);
	}

	addr = node | (area << 10);


	printf("AA:00:04:00:%02X:%02X%s", (int)(addr & 0xff), 
					(int)(addr >> 8),
					(newline) ? "\n" : "");

	return EX_OK;
}

